Struct DateSeries

struct DateSeries { ... }

An iterator over periodic dates, created by Date::series.

It is exhausted when the next value would exceed a Span or Date value.

Implementations

impl Clone for DateSeries

fn clone(self: &Self) -> DateSeries

impl Debug for DateSeries

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Freeze for DateSeries

impl Iterator for DateSeries

fn next(self: &mut Self) -> Option<Date>

impl RefUnwindSafe for DateSeries

impl Send for DateSeries

impl Sync for DateSeries

impl Unpin for DateSeries

impl UnsafeUnpin for DateSeries

impl UnwindSafe for DateSeries

impl<I> IntoIterator for DateSeries

fn into_iter(self: Self) -> I

impl<T> Any for DateSeries

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for DateSeries

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for DateSeries

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for DateSeries

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for DateSeries

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for DateSeries

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for DateSeries

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for DateSeries

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for DateSeries

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>