Struct DateTimeRound

struct DateTimeRound { ... }

Options for DateTime::round.

This type provides a way to configure the rounding of a civil datetime. In particular, DateTime::round accepts anything that implements the Into<DateTimeRound> trait. There are some trait implementations that therefore make calling DateTime::round in some common cases more ergonomic:

Note that in the default configuration, no rounding occurs.

Example

This example shows how to round a datetime to the nearest second:

use jiff::{civil::{DateTime, date}, Unit};

let dt: DateTime = "2024-06-20 16:24:59.5".parse()?;
assert_eq!(
    dt.round(Unit::Second)?,
    // The second rounds up and causes minutes to increase.
    date(2024, 6, 20).at(16, 25, 0, 0),
);

# Ok::<(), Box<dyn std::error::Error>>(())

The above makes use of the fact that Unit implements Into<DateTimeRound>. If you want to change the rounding mode to, say, truncation, then you'll need to construct a DateTimeRound explicitly since there are no convenience Into trait implementations for RoundMode.

use jiff::{civil::{DateTime, DateTimeRound, date}, RoundMode, Unit};

let dt: DateTime = "2024-06-20 16:24:59.5".parse()?;
assert_eq!(
    dt.round(
        DateTimeRound::new().smallest(Unit::Second).mode(RoundMode::Trunc),
    )?,
    // The second just gets truncated as if it wasn't there.
    date(2024, 6, 20).at(16, 24, 59, 0),
);

# Ok::<(), Box<dyn std::error::Error>>(())

Implementations

impl DateTimeRound

fn new() -> DateTimeRound

Create a new default configuration for rounding a DateTime.

fn smallest(self: Self, unit: Unit) -> DateTimeRound

Set the smallest units allowed in the datetime returned after rounding.

Any units below the smallest configured unit will be used, along with the rounding increment and rounding mode, to determine the value of the smallest unit. For example, when rounding 2024-06-20T03:25:30 to the nearest minute, the 30 second unit will result in rounding the minute unit of 25 up to 26 and zeroing out everything below minutes.

This defaults to Unit::Nanosecond.

Errors

The smallest units must be no greater than Unit::Day. And when the smallest unit is Unit::Day, the rounding increment must be equal to 1. Otherwise an error will be returned from DateTime::round.

Example

use jiff::{civil::{DateTimeRound, date}, Unit};

let dt = date(2024, 6, 20).at(3, 25, 30, 0);
assert_eq!(
    dt.round(DateTimeRound::new().smallest(Unit::Minute))?,
    date(2024, 6, 20).at(3, 26, 0, 0),
);
// Or, utilize the `From<Unit> for DateTimeRound` impl:
assert_eq!(
    dt.round(Unit::Minute)?,
    date(2024, 6, 20).at(3, 26, 0, 0),
);

# Ok::<(), Box<dyn std::error::Error>>(())
fn mode(self: Self, mode: RoundMode) -> DateTimeRound

Set the rounding mode.

This defaults to RoundMode::HalfExpand, which rounds away from zero. It matches the kind of rounding you might have been taught in school.

Example

This shows how to always round datetimes up towards positive infinity.

use jiff::{civil::{DateTime, DateTimeRound, date}, RoundMode, Unit};

let dt: DateTime = "2024-06-20 03:25:01".parse()?;
assert_eq!(
    dt.round(
        DateTimeRound::new()
            .smallest(Unit::Minute)
            .mode(RoundMode::Ceil),
    )?,
    date(2024, 6, 20).at(3, 26, 0, 0),
);

# Ok::<(), Box<dyn std::error::Error>>(())
fn increment(self: Self, increment: i64) -> DateTimeRound

Set the rounding increment for the smallest unit.

The default value is 1. Other values permit rounding the smallest unit to the nearest integer increment specified. For example, if the smallest unit is set to Unit::Minute, then a rounding increment of 30 would result in rounding in increments of a half hour. That is, the only minute value that could result would be 0 or 30.

Errors

When the smallest unit is Unit::Day, then the rounding increment must be 1 or else DateTime::round will return an error.

For other units, the rounding increment must divide evenly into the next highest unit above the smallest unit set. The rounding increment must also not be equal to the next highest unit. For example, if the smallest unit is Unit::Nanosecond, then some of the valid values for the rounding increment are 1, 2, 4, 5, 100 and 500. Namely, any integer that divides evenly into 1,000 nanoseconds since there are 1,000 nanoseconds in the next highest unit (microseconds).

Example

This example shows how to round a datetime to the nearest 10 minute increment.

use jiff::{civil::{DateTime, DateTimeRound, date}, RoundMode, Unit};

let dt: DateTime = "2024-06-20 03:24:59".parse()?;
assert_eq!(
    dt.round((Unit::Minute, 10))?,
    date(2024, 6, 20).at(3, 20, 0, 0),
);

# Ok::<(), Box<dyn std::error::Error>>(())

impl Clone for DateTimeRound

fn clone(self: &Self) -> DateTimeRound

impl Copy for DateTimeRound

impl Debug for DateTimeRound

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Default for DateTimeRound

fn default() -> DateTimeRound

impl Freeze for DateTimeRound

impl From for DateTimeRound

fn from(unit: Unit) -> DateTimeRound

impl From for DateTimeRound

fn from((unit, increment): (Unit, i64)) -> DateTimeRound

impl RefUnwindSafe for DateTimeRound

impl Send for DateTimeRound

impl Sync for DateTimeRound

impl Unpin for DateTimeRound

impl UnsafeUnpin for DateTimeRound

impl UnwindSafe for DateTimeRound

impl<T> Any for DateTimeRound

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for DateTimeRound

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for DateTimeRound

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for DateTimeRound

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for DateTimeRound

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for DateTimeRound

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for DateTimeRound

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for DateTimeRound

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for DateTimeRound

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>