Struct WeekdaysForward
struct WeekdaysForward { ... }
An unending iterator of the days of the week.
This iterator is created by calling Weekday::cycle_forward.
Implementations
impl Clone for WeekdaysForward
fn clone(self: &Self) -> WeekdaysForward
impl Debug for WeekdaysForward
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Freeze for WeekdaysForward
impl FusedIterator for WeekdaysForward
impl Iterator for WeekdaysForward
fn next(self: &mut Self) -> Option<Weekday>
impl RefUnwindSafe for WeekdaysForward
impl Send for WeekdaysForward
impl Sync for WeekdaysForward
impl Unpin for WeekdaysForward
impl UnsafeUnpin for WeekdaysForward
impl UnwindSafe for WeekdaysForward
impl<I> IntoIterator for WeekdaysForward
fn into_iter(self: Self) -> I
impl<T> Any for WeekdaysForward
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for WeekdaysForward
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for WeekdaysForward
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for WeekdaysForward
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for WeekdaysForward
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for WeekdaysForward
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for WeekdaysForward
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for WeekdaysForward
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for WeekdaysForward
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>