Struct WeekdaysReverse

struct WeekdaysReverse { ... }

An unending iterator of the days of the week in reverse.

This iterator is created by calling Weekday::cycle_reverse.

Implementations

impl Clone for WeekdaysReverse

fn clone(self: &Self) -> WeekdaysReverse

impl Debug for WeekdaysReverse

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Freeze for WeekdaysReverse

impl FusedIterator for WeekdaysReverse

impl Iterator for WeekdaysReverse

fn next(self: &mut Self) -> Option<Weekday>

impl RefUnwindSafe for WeekdaysReverse

impl Send for WeekdaysReverse

impl Sync for WeekdaysReverse

impl Unpin for WeekdaysReverse

impl UnsafeUnpin for WeekdaysReverse

impl UnwindSafe for WeekdaysReverse

impl<I> IntoIterator for WeekdaysReverse

fn into_iter(self: Self) -> I

impl<T> Any for WeekdaysReverse

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for WeekdaysReverse

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for WeekdaysReverse

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for WeekdaysReverse

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for WeekdaysReverse

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for WeekdaysReverse

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for WeekdaysReverse

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for WeekdaysReverse

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for WeekdaysReverse

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>