Struct SpanCompare

struct SpanCompare<'a> { ... }

Options for Span::compare.

This type provides a way to ergonomically compare two spans with an optional relative datetime. Namely, a relative datetime is only needed when at least one of the two spans being compared has a non-zero calendar unit (years, months, weeks or days). Otherwise, an error will be returned.

Callers may use SpanCompare::days_are_24_hours to opt into 24-hour invariant days (and 7-day weeks) without providing a relative datetime.

The main way to construct values of this type is with its From trait implementations:

Example

use jiff::ToSpan;

let span1 = 3.hours();
let span2 = 180.minutes();
assert_eq!(span1.compare(span2)?, std::cmp::Ordering::Equal);

# Ok::<(), Box<dyn std::error::Error>>(())

Implementations

impl<'a> SpanCompare<'a>

fn days_are_24_hours(self: Self) -> SpanCompare<'a>

This is a convenience function for setting the relative option on this configuration to SpanRelativeTo::days_are_24_hours.

Example

When comparing spans involving days, either a relative datetime must be provided, or a special assertion opting into 24-hour days is required. Otherwise, you get an error.

use jiff::{SpanCompare, ToSpan, Unit};

let span1 = 2.days().hours(12);
let span2 = 60.hours();
// No relative date provided, which results in an error.
assert_eq!(
    span1.compare(span2).unwrap_err().to_string(),
    "using unit 'day' in a span or configuration requires that \
     either a relative reference time be given or \
     `SpanRelativeTo::days_are_24_hours()` is used to indicate \
     invariant 24-hour days, but neither were provided",
);
let ordering = span1.compare(
    SpanCompare::from(span2).days_are_24_hours(),
)?;
assert_eq!(ordering, std::cmp::Ordering::Equal);

# Ok::<(), Box<dyn std::error::Error>>(())

impl From for SpanCompare<'static>

fn from((span, date): (Span, Date)) -> SpanCompare<'static>

impl From for SpanCompare<'static>

fn from(span: Span) -> SpanCompare<'static>

impl From for SpanCompare<'static>

fn from((span, datetime): (Span, DateTime)) -> SpanCompare<'static>

impl<'a> Clone for SpanCompare<'a>

fn clone(self: &Self) -> SpanCompare<'a>

impl<'a> Copy for SpanCompare<'a>

impl<'a> Debug for SpanCompare<'a>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'a> Freeze for SpanCompare<'a>

impl<'a> From for SpanCompare<'a>

fn from((span, zoned): (Span, &'a Zoned)) -> SpanCompare<'a>

impl<'a> From for SpanCompare<'a>

fn from((span, relative): (Span, SpanRelativeTo<'a>)) -> SpanCompare<'a>

impl<'a> From for SpanCompare<'static>

fn from(span: &'a Span) -> SpanCompare<'static>

impl<'a> From for SpanCompare<'static>

fn from((span, date): (&'a Span, Date)) -> SpanCompare<'static>

impl<'a> From for SpanCompare<'static>

fn from((span, datetime): (&'a Span, DateTime)) -> SpanCompare<'static>

impl<'a> RefUnwindSafe for SpanCompare<'a>

impl<'a> Send for SpanCompare<'a>

impl<'a> Sync for SpanCompare<'a>

impl<'a> Unpin for SpanCompare<'a>

impl<'a> UnsafeUnpin for SpanCompare<'a>

impl<'a> UnwindSafe for SpanCompare<'a>

impl<'a, 'b> From for SpanCompare<'b>

fn from((span, relative): (&'a Span, SpanRelativeTo<'b>)) -> SpanCompare<'b>

impl<'a, 'b> From for SpanCompare<'b>

fn from((span, zoned): (&'a Span, &'b Zoned)) -> SpanCompare<'b>

impl<T> Any for SpanCompare<'a>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for SpanCompare<'a>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for SpanCompare<'a>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for SpanCompare<'a>

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for SpanCompare<'a>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for SpanCompare<'a>

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for SpanCompare<'a>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for SpanCompare<'a>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for SpanCompare<'a>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>