Struct TimestampDisplayWithOffset

struct TimestampDisplayWithOffset { ... }

A type for formatting a Timestamp with a specific offset.

This type is created by the Timestamp::display_with_offset method.

Like the std::fmt::Display trait implementation for Timestamp, this always emits an RFC 3339 compliant string. Unlike Timestamp's Display trait implementation, which always uses Z or "Zulu" time, this always uses an offfset.

Forrmatting options supported

Example

use jiff::{tz, Timestamp};

let offset = tz::offset(-5);
let ts = Timestamp::new(1_123_456_789, 123_000_000)?;
assert_eq!(
    format!("{ts:.6}", ts = ts.display_with_offset(offset)),
    "2005-08-07T18:19:49.123000-05:00",
);
// Precision values greater than 9 are clamped to 9.
assert_eq!(
    format!("{ts:.300}", ts = ts.display_with_offset(offset)),
    "2005-08-07T18:19:49.123000000-05:00",
);
// A precision of 0 implies the entire fractional
// component is always truncated.
assert_eq!(
    format!("{ts:.0}", ts = ts.display_with_offset(tz::Offset::UTC)),
    "2005-08-07T23:19:49+00:00",
);

# Ok::<(), Box<dyn std::error::Error>>(())

Implementations

impl Clone for TimestampDisplayWithOffset

fn clone(self: &Self) -> TimestampDisplayWithOffset

impl Copy for TimestampDisplayWithOffset

impl Debug for TimestampDisplayWithOffset

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Display for TimestampDisplayWithOffset

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Freeze for TimestampDisplayWithOffset

impl RefUnwindSafe for TimestampDisplayWithOffset

impl Send for TimestampDisplayWithOffset

impl Sync for TimestampDisplayWithOffset

impl Unpin for TimestampDisplayWithOffset

impl UnsafeUnpin for TimestampDisplayWithOffset

impl UnwindSafe for TimestampDisplayWithOffset

impl<T> Any for TimestampDisplayWithOffset

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for TimestampDisplayWithOffset

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for TimestampDisplayWithOffset

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for TimestampDisplayWithOffset

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for TimestampDisplayWithOffset

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for TimestampDisplayWithOffset

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T> ToString for TimestampDisplayWithOffset

fn to_string(self: &Self) -> String

impl<T, U> Into for TimestampDisplayWithOffset

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for TimestampDisplayWithOffset

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for TimestampDisplayWithOffset

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>