Struct TimestampSeries
struct TimestampSeries { ... }
An iterator over periodic timestamps, created by Timestamp::series.
It is exhausted when the next value would exceed a Span or
Timestamp value.
Implementations
impl Clone for TimestampSeries
fn clone(self: &Self) -> TimestampSeries
impl Debug for TimestampSeries
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Freeze for TimestampSeries
impl Iterator for TimestampSeries
fn next(self: &mut Self) -> Option<Timestamp>
impl RefUnwindSafe for TimestampSeries
impl Send for TimestampSeries
impl Sync for TimestampSeries
impl Unpin for TimestampSeries
impl UnsafeUnpin for TimestampSeries
impl UnwindSafe for TimestampSeries
impl<I> IntoIterator for TimestampSeries
fn into_iter(self: Self) -> I
impl<T> Any for TimestampSeries
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for TimestampSeries
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for TimestampSeries
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for TimestampSeries
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for TimestampSeries
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for TimestampSeries
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for TimestampSeries
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for TimestampSeries
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for TimestampSeries
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>