Struct TimeZoneName

struct TimeZoneName<'d> { ... }

A name for a time zone yield by the TimeZoneNameIter iterator.

The iterator is created by TimeZoneDatabase::available.

The lifetime parameter corresponds to the lifetime of the TimeZoneDatabase from which this name was created.

Implementations

impl<'d> TimeZoneName<'d>

fn as_str<'a>(self: &'a Self) -> &'a str

Returns this time zone name as a borrowed string.

Note that the lifetime of the string returned is tied to self, which may be shorter than the lifetime 'd of the originating TimeZoneDatabase.

impl<'d> Clone for TimeZoneName<'d>

fn clone(self: &Self) -> TimeZoneName<'d>

impl<'d> Debug for TimeZoneName<'d>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'d> Display for TimeZoneName<'d>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'d> Eq for TimeZoneName<'d>

impl<'d> Freeze for TimeZoneName<'d>

impl<'d> Hash for TimeZoneName<'d>

fn hash<__H: $crate::hash::Hasher>(self: &Self, state: &mut __H)

impl<'d> Ord for TimeZoneName<'d>

fn cmp(self: &Self, other: &TimeZoneName<'d>) -> Ordering

impl<'d> PartialEq for TimeZoneName<'d>

fn eq(self: &Self, other: &TimeZoneName<'d>) -> bool

impl<'d> PartialOrd for TimeZoneName<'d>

fn partial_cmp(self: &Self, other: &TimeZoneName<'d>) -> Option<Ordering>

impl<'d> RefUnwindSafe for TimeZoneName<'d>

impl<'d> Send for TimeZoneName<'d>

impl<'d> StructuralPartialEq for TimeZoneName<'d>

impl<'d> Sync for TimeZoneName<'d>

impl<'d> Unpin for TimeZoneName<'d>

impl<'d> UnsafeUnpin for TimeZoneName<'d>

impl<'d> UnwindSafe for TimeZoneName<'d>

impl<T> Any for TimeZoneName<'d>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for TimeZoneName<'d>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for TimeZoneName<'d>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for TimeZoneName<'d>

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for TimeZoneName<'d>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for TimeZoneName<'d>

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T> ToString for TimeZoneName<'d>

fn to_string(self: &Self) -> String

impl<T, U> Into for TimeZoneName<'d>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for TimeZoneName<'d>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for TimeZoneName<'d>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>