Struct TimeZoneNameIter
struct TimeZoneNameIter<'d> { ... }
An iterator over the time zone identifiers in a TimeZoneDatabase.
This iterator is created by TimeZoneDatabase::available.
There are no guarantees about the order in which this iterator yields time zone identifiers.
The lifetime parameter corresponds to the lifetime of the
TimeZoneDatabase from which this iterator was created.
Implementations
impl<'d> Clone for TimeZoneNameIter<'d>
fn clone(self: &Self) -> TimeZoneNameIter<'d>
impl<'d> Debug for TimeZoneNameIter<'d>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'d> Freeze for TimeZoneNameIter<'d>
impl<'d> Iterator for TimeZoneNameIter<'d>
fn next(self: &mut Self) -> Option<TimeZoneName<'d>>
impl<'d> RefUnwindSafe for TimeZoneNameIter<'d>
impl<'d> Send for TimeZoneNameIter<'d>
impl<'d> Sync for TimeZoneNameIter<'d>
impl<'d> Unpin for TimeZoneNameIter<'d>
impl<'d> UnsafeUnpin for TimeZoneNameIter<'d>
impl<'d> UnwindSafe for TimeZoneNameIter<'d>
impl<I> IntoIterator for TimeZoneNameIter<'d>
fn into_iter(self: Self) -> I
impl<T> Any for TimeZoneNameIter<'d>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for TimeZoneNameIter<'d>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for TimeZoneNameIter<'d>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for TimeZoneNameIter<'d>
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for TimeZoneNameIter<'d>
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for TimeZoneNameIter<'d>
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for TimeZoneNameIter<'d>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for TimeZoneNameIter<'d>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for TimeZoneNameIter<'d>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>