Struct DateTimeDifference

struct DateTimeDifference { ... }

Options for DateTime::since and DateTime::until.

This type provides a way to configure the calculation of spans between two DateTime values. In particular, both DateTime::since and DateTime::until accept anything that implements Into<DateTimeDifference>. There are a few key trait implementations that make this convenient:

One can also provide a DateTimeDifference value directly. Doing so is necessary to use the rounding features of calculating a span. For example, setting the smallest unit (defaults to Unit::Nanosecond), the rounding mode (defaults to RoundMode::Trunc) and the rounding increment (defaults to 1). The defaults are selected such that no rounding occurs.

Rounding a span as part of calculating it is provided as a convenience. Callers may choose to round the span as a distinct step via Span::round, but callers may need to provide a reference date for rounding larger units. By coupling rounding with routines like DateTime::since, the reference date can be set automatically based on the input to DateTime::since.

Example

This example shows how to round a span between two datetimes to the nearest half-hour, with ties breaking away from zero.

use jiff::{civil::{DateTime, DateTimeDifference}, RoundMode, ToSpan, Unit};

let dt1 = "2024-03-15 08:14:00.123456789".parse::<DateTime>()?;
let dt2 = "2030-03-22 15:00".parse::<DateTime>()?;
let span = dt1.until(
    DateTimeDifference::new(dt2)
        .smallest(Unit::Minute)
        .largest(Unit::Year)
        .mode(RoundMode::HalfExpand)
        .increment(30),
)?;
assert_eq!(span, 6.years().days(7).hours(7).fieldwise());

# Ok::<(), Box<dyn std::error::Error>>(())

Implementations

impl DateTimeDifference

fn new(datetime: DateTime) -> DateTimeDifference

Create a new default configuration for computing the span between the given datetime and some other datetime (specified as the receiver in DateTime::since or DateTime::until).

fn smallest(self: Self, unit: Unit) -> DateTimeDifference

Set the smallest units allowed in the span returned.

When a largest unit is not specified and the smallest unit is days or greater, then the largest unit is automatically set to be equal to the smallest unit.

Errors

The smallest units must be no greater than the largest units. If this is violated, then computing a span with this configuration will result in an error.

Example

This shows how to round a span between two datetimes to the nearest number of weeks.

use jiff::{
    civil::{DateTime, DateTimeDifference},
    RoundMode, ToSpan, Unit,
};

let dt1 = "2024-03-15 08:14".parse::<DateTime>()?;
let dt2 = "2030-11-22 08:30".parse::<DateTime>()?;
let span = dt1.until(
    DateTimeDifference::new(dt2)
        .smallest(Unit::Week)
        .largest(Unit::Week)
        .mode(RoundMode::HalfExpand),
)?;
assert_eq!(span, 349.weeks().fieldwise());

# Ok::<(), Box<dyn std::error::Error>>(())
fn largest(self: Self, unit: Unit) -> DateTimeDifference

Set the largest units allowed in the span returned.

When a largest unit is not specified and the smallest unit is days or greater, then the largest unit is automatically set to be equal to the smallest unit. Otherwise, when the largest unit is not specified, it is set to days.

Once a largest unit is set, there is no way to change this rounding configuration back to using the "automatic" default. Instead, callers must create a new configuration.

Errors

The largest units, when set, must be at least as big as the smallest units (which defaults to Unit::Nanosecond). If this is violated, then computing a span with this configuration will result in an error.

Example

This shows how to round a span between two datetimes to units no bigger than seconds.

use jiff::{civil::{DateTime, DateTimeDifference}, ToSpan, Unit};

let dt1 = "2024-03-15 08:14".parse::<DateTime>()?;
let dt2 = "2030-11-22 08:30".parse::<DateTime>()?;
let span = dt1.until(
    DateTimeDifference::new(dt2).largest(Unit::Second),
)?;
assert_eq!(span, 211076160.seconds().fieldwise());

# Ok::<(), Box<dyn std::error::Error>>(())
fn mode(self: Self, mode: RoundMode) -> DateTimeDifference

Set the rounding mode.

This defaults to RoundMode::Trunc since it's plausible that rounding "up" in the context of computing the span between two datetimes could be surprising in a number of cases. The RoundMode::HalfExpand mode corresponds to typical rounding you might have learned about in school. But a variety of other rounding modes exist.

Example

This shows how to always round "up" towards positive infinity.

use jiff::{
    civil::{DateTime, DateTimeDifference},
    RoundMode, ToSpan, Unit,
};

let dt1 = "2024-03-15 08:10".parse::<DateTime>()?;
let dt2 = "2024-03-15 08:11".parse::<DateTime>()?;
let span = dt1.until(
    DateTimeDifference::new(dt2)
        .smallest(Unit::Hour)
        .mode(RoundMode::Ceil),
)?;
// Only one minute elapsed, but we asked to always round up!
assert_eq!(span, 1.hour().fieldwise());

// Since `Ceil` always rounds toward positive infinity, the behavior
// flips for a negative span.
let span = dt1.since(
    DateTimeDifference::new(dt2)
        .smallest(Unit::Hour)
        .mode(RoundMode::Ceil),
)?;
assert_eq!(span, 0.hour().fieldwise());

# Ok::<(), Box<dyn std::error::Error>>(())
fn increment(self: Self, increment: i64) -> DateTimeDifference

Set the rounding increment for the smallest unit.

The default value is 1. Other values permit rounding the smallest unit to the nearest integer increment specified. For example, if the smallest unit is set to Unit::Minute, then a rounding increment of 30 would result in rounding in increments of a half hour. That is, the only minute value that could result would be 0 or 30.

Errors

When the smallest unit is less than days, the rounding increment must divide evenly into the next highest unit after the smallest unit configured (and must not be equivalent to it). For example, if the smallest unit is Unit::Nanosecond, then some of the valid values for the rounding increment are 1, 2, 4, 5, 100 and 500. Namely, any integer that divides evenly into 1,000 nanoseconds since there are 1,000 nanoseconds in the next highest unit (microseconds).

The error will occur when computing the span, and not when setting the increment here.

Example

This shows how to round the span between two datetimes to the nearest 5 minute increment.

use jiff::{
    civil::{DateTime, DateTimeDifference},
    RoundMode, ToSpan, Unit,
};

let dt1 = "2024-03-15 08:19".parse::<DateTime>()?;
let dt2 = "2024-03-15 12:52".parse::<DateTime>()?;
let span = dt1.until(
    DateTimeDifference::new(dt2)
        .smallest(Unit::Minute)
        .increment(5)
        .mode(RoundMode::HalfExpand),
)?;
assert_eq!(span, 4.hour().minutes(35).fieldwise());

# Ok::<(), Box<dyn std::error::Error>>(())

impl Clone for DateTimeDifference

fn clone(self: &Self) -> DateTimeDifference

impl Copy for DateTimeDifference

impl Debug for DateTimeDifference

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Freeze for DateTimeDifference

impl From for DateTimeDifference

fn from(date: Date) -> DateTimeDifference

impl From for DateTimeDifference

fn from((largest, zdt): (Unit, Zoned)) -> DateTimeDifference

impl From for DateTimeDifference

fn from(zdt: Zoned) -> DateTimeDifference

impl From for DateTimeDifference

fn from((largest, dt): (Unit, DateTime)) -> DateTimeDifference

impl From for DateTimeDifference

fn from(dt: DateTime) -> DateTimeDifference

impl From for DateTimeDifference

fn from((largest, date): (Unit, Date)) -> DateTimeDifference

impl RefUnwindSafe for DateTimeDifference

impl Send for DateTimeDifference

impl Sync for DateTimeDifference

impl Unpin for DateTimeDifference

impl UnsafeUnpin for DateTimeDifference

impl UnwindSafe for DateTimeDifference

impl<'a> From for DateTimeDifference

fn from((largest, zdt): (Unit, &'a Zoned)) -> DateTimeDifference

impl<'a> From for DateTimeDifference

fn from(zdt: &'a Zoned) -> DateTimeDifference

impl<T> Any for DateTimeDifference

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for DateTimeDifference

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for DateTimeDifference

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for DateTimeDifference

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for DateTimeDifference

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for DateTimeDifference

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for DateTimeDifference

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for DateTimeDifference

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for DateTimeDifference

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>