Struct TimeDifference
struct TimeDifference { ... }
Options for Time::since and Time::until.
This type provides a way to configure the calculation of spans between two
Time values. In particular, both Time::since and Time::until accept
anything that implements Into<TimeDifference>. There are a few key trait
implementations that make this convenient:
From<Time> for TimeDifferencewill construct a configuration consisting of just the time. So for example,time1.until(time2)will return the span fromtime1totime2.From<DateTime> for TimeDifferencewill construct a configuration consisting of just the time from the given datetime. So for example,time.since(datetime)returns the span fromdatetime.time()totime.From<(Unit, Time)>is a convenient way to specify the largest units that should be present on the span returned. By default, the largest units are hours. Using this trait implementation is equivalent toTimeDifference::new(time).largest(unit).From<(Unit, DateTime)>is like the one above, but with the time from the given datetime.
One can also provide a TimeDifference value directly. Doing so
is necessary to use the rounding features of calculating a span. For
example, setting the smallest unit (defaults to Unit::Nanosecond), the
rounding mode (defaults to RoundMode::Trunc) and the rounding increment
(defaults to 1). The defaults are selected such that no rounding occurs.
Rounding a span as part of calculating it is provided as a convenience.
Callers may choose to round the span as a distinct step via
Span::round.
Example
This example shows how to round a span between two datetimes to the nearest half-hour, with ties breaking away from zero.
use ;
let t1 = "08:14:00.123456789".?;
let t2 = "15:00".?;
let span = t1.until?;
assert_eq!;
// One less minute, and because of the HalfExpand mode, the span would
// get rounded down.
let t2 = "14:59".?;
let span = t1.until?;
assert_eq!;
# Ok::
Implementations
impl TimeDifference
fn new(time: Time) -> TimeDifferenceCreate a new default configuration for computing the span between the given time and some other time (specified as the receiver in
Time::sinceorTime::until).fn smallest(self: Self, unit: Unit) -> TimeDifferenceSet the smallest units allowed in the span returned.
Errors
The smallest units must be no greater than the largest units. If this is violated, then computing a span with this configuration will result in an error.
Example
This shows how to round a span between two times to units no less than seconds.
use ; let t1 = "08:14:02.5001".?; let t2 = "08:30:03.0001".?; let span = t1.until?; assert_eq!; # Ok::fn largest(self: Self, unit: Unit) -> TimeDifferenceSet the largest units allowed in the span returned.
When a largest unit is not specified, computing a span between times behaves as if it were set to
Unit::Hour.Errors
The largest units, when set, must be at least as big as the smallest units (which defaults to
Unit::Nanosecond). If this is violated, then computing a span with this configuration will result in an error.Example
This shows how to round a span between two times to units no bigger than seconds.
use ; let t1 = "08:14".?; let t2 = "08:30".?; let span = t1.until?; assert_eq!; # Ok::fn mode(self: Self, mode: RoundMode) -> TimeDifferenceSet the rounding mode.
This defaults to
RoundMode::Truncsince it's plausible that rounding "up" in the context of computing the span between two times could be surprising in a number of cases. TheRoundMode::HalfExpandmode corresponds to typical rounding you might have learned about in school. But a variety of other rounding modes exist.Example
This shows how to always round "up" towards positive infinity.
use ; let t1 = "08:10".?; let t2 = "08:11".?; let span = t1.until?; // Only one minute elapsed, but we asked to always round up! assert_eq!; // Since `Ceil` always rounds toward positive infinity, the behavior // flips for a negative span. let span = t1.since?; assert_eq!; # Ok::fn increment(self: Self, increment: i64) -> TimeDifferenceSet the rounding increment for the smallest unit.
The default value is
1. Other values permit rounding the smallest unit to the nearest integer increment specified. For example, if the smallest unit is set toUnit::Minute, then a rounding increment of30would result in rounding in increments of a half hour. That is, the only minute value that could result would be0or30.Errors
The rounding increment must divide evenly into the next highest unit after the smallest unit configured (and must not be equivalent to it). For example, if the smallest unit is
Unit::Nanosecond, then some of the valid values for the rounding increment are1,2,4,5,100and500. Namely, any integer that divides evenly into1,000nanoseconds since there are1,000nanoseconds in the next highest unit (microseconds).The error will occur when computing the span, and not when setting the increment here.
Example
This shows how to round the span between two times to the nearest 5 minute increment.
use ; let t1 = "08:19".?; let t2 = "12:52".?; let span = t1.until?; assert_eq!; # Ok::
impl Clone for TimeDifference
fn clone(self: &Self) -> TimeDifference
impl Copy for TimeDifference
impl Debug for TimeDifference
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Freeze for TimeDifference
impl From for TimeDifference
fn from(dt: DateTime) -> TimeDifference
impl From for TimeDifference
fn from((largest, time): (Unit, Time)) -> TimeDifference
impl From for TimeDifference
fn from((largest, dt): (Unit, DateTime)) -> TimeDifference
impl From for TimeDifference
fn from((largest, zdt): (Unit, Zoned)) -> TimeDifference
impl From for TimeDifference
fn from(time: Time) -> TimeDifference
impl From for TimeDifference
fn from(zdt: Zoned) -> TimeDifference
impl RefUnwindSafe for TimeDifference
impl Send for TimeDifference
impl Sync for TimeDifference
impl Unpin for TimeDifference
impl UnsafeUnpin for TimeDifference
impl UnwindSafe for TimeDifference
impl<'a> From for TimeDifference
fn from((largest, zdt): (Unit, &'a Zoned)) -> TimeDifference
impl<'a> From for TimeDifference
fn from(zdt: &'a Zoned) -> TimeDifference
impl<T> Any for TimeDifference
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for TimeDifference
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for TimeDifference
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for TimeDifference
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for TimeDifference
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for TimeDifference
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for TimeDifference
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for TimeDifference
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for TimeDifference
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>