Struct TimeZoneAnnotationName
struct TimeZoneAnnotationName<'n> { ... }
A time zone annotation parsed from a datetime string.
By default, a time zone annotation name borrows its name from the
input it was parsed from. When the alloc feature is enabled,
callers can de-couple the annotation from the parsed input with
TimeZoneAnnotationName::into_owned.
A value of this type is usually found via Pieces::time_zone_annotation,
but callers can also construct one via this type's From<&str> trait
implementation if necessary.
Implementations
impl<'n> TimeZoneAnnotationName<'n>
fn as_str<'a>(self: &'a Self) -> &'a strReturns the name of this time zone annotation as a string slice.
Note that the lifetime of the string slice returned is tied to the lifetime of this time zone annotation. This may be shorter than the lifetime of the string,
'n, in this annotation.fn into_owned(self: Self) -> TimeZoneAnnotationName<'static>Converts this time zone annotation name into an "owned" value whose lifetime is
'static.If this was already an "owned" value, then this is a no-op.
impl<'n> Clone for TimeZoneAnnotationName<'n>
fn clone(self: &Self) -> TimeZoneAnnotationName<'n>
impl<'n> Debug for TimeZoneAnnotationName<'n>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'n> Eq for TimeZoneAnnotationName<'n>
impl<'n> Freeze for TimeZoneAnnotationName<'n>
impl<'n> From for TimeZoneAnnotationName<'n>
fn from(string: &'n str) -> TimeZoneAnnotationName<'n>
impl<'n> Hash for TimeZoneAnnotationName<'n>
fn hash<__H: $crate::hash::Hasher>(self: &Self, state: &mut __H)
impl<'n> PartialEq for TimeZoneAnnotationName<'n>
fn eq(self: &Self, other: &TimeZoneAnnotationName<'n>) -> bool
impl<'n> RefUnwindSafe for TimeZoneAnnotationName<'n>
impl<'n> Send for TimeZoneAnnotationName<'n>
impl<'n> StructuralPartialEq for TimeZoneAnnotationName<'n>
impl<'n> Sync for TimeZoneAnnotationName<'n>
impl<'n> Unpin for TimeZoneAnnotationName<'n>
impl<'n> UnsafeUnpin for TimeZoneAnnotationName<'n>
impl<'n> UnwindSafe for TimeZoneAnnotationName<'n>
impl<T> Any for TimeZoneAnnotationName<'n>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for TimeZoneAnnotationName<'n>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for TimeZoneAnnotationName<'n>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for TimeZoneAnnotationName<'n>
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for TimeZoneAnnotationName<'n>
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for TimeZoneAnnotationName<'n>
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for TimeZoneAnnotationName<'n>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for TimeZoneAnnotationName<'n>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for TimeZoneAnnotationName<'n>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>