Struct SpanArithmetic
struct SpanArithmetic<'a> { ... }
Options for Span::checked_add and Span::checked_sub.
This type provides a way to ergonomically add two spans with an optional relative datetime. Namely, a relative datetime is only needed when at least one of the two spans being added (or subtracted) has a non-zero calendar unit (years, months, weeks or days). Otherwise, an error will be returned.
Callers may use SpanArithmetic::days_are_24_hours to opt into 24-hour
invariant days (and 7-day weeks) without providing a relative datetime.
The main way to construct values of this type is with its From trait
implementations:
From<Span> for SpanArithmeticadds (or subtracts) the given span to the receiver inSpan::checked_add(orSpan::checked_sub).From<(Span, civil::Date)> for SpanArithmeticadds (or subtracts) the given span to the receiver inSpan::checked_add(orSpan::checked_sub), relative to the given date. There are alsoFromimplementations forcivil::DateTime,ZonedandSpanRelativeTo.
Example
use ToSpan;
assert_eq!;
# Ok::
Implementations
impl<'a> SpanArithmetic<'a>
fn days_are_24_hours(self: Self) -> SpanArithmetic<'a>This is a convenience function for setting the relative option on this configuration to
SpanRelativeTo::days_are_24_hours.Example
When doing arithmetic on spans involving days, either a relative datetime must be provided, or a special assertion opting into 24-hour days is required. Otherwise, you get an error.
use ; let span1 = 2.days.hours; let span2 = 12.hours; // No relative date provided, which results in an error. assert_eq!; let sum = span1.checked_add?; assert_eq!; # Ok::
impl From for SpanArithmetic<'static>
fn from((duration, date): (SignedDuration, Date)) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from((duration, datetime): (UnsignedDuration, DateTime)) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from(duration: SignedDuration) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from((duration, datetime): (SignedDuration, DateTime)) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from((span, date): (Span, Date)) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from(span: Span) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from((span, datetime): (Span, DateTime)) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from(duration: UnsignedDuration) -> SpanArithmetic<'static>
impl From for SpanArithmetic<'static>
fn from((duration, date): (UnsignedDuration, Date)) -> SpanArithmetic<'static>
impl<'a> Clone for SpanArithmetic<'a>
fn clone(self: &Self) -> SpanArithmetic<'a>
impl<'a> Copy for SpanArithmetic<'a>
impl<'a> Debug for SpanArithmetic<'a>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'a> Freeze for SpanArithmetic<'a>
impl<'a> From for SpanArithmetic<'a>
fn from((span, relative): (Span, SpanRelativeTo<'a>)) -> SpanArithmetic<'a>
impl<'a> From for SpanArithmetic<'a>
fn from((duration, zoned): (UnsignedDuration, &'a Zoned)) -> SpanArithmetic<'a>
impl<'a> From for SpanArithmetic<'a>
fn from((duration, zoned): (SignedDuration, &'a Zoned)) -> SpanArithmetic<'a>
impl<'a> From for SpanArithmetic<'a>
fn from((span, zoned): (Span, &'a Zoned)) -> SpanArithmetic<'a>
impl<'a> From for SpanArithmetic<'static>
fn from((span, date): (&'a Span, Date)) -> SpanArithmetic<'static>
impl<'a> From for SpanArithmetic<'static>
fn from((span, datetime): (&'a Span, DateTime)) -> SpanArithmetic<'static>
impl<'a> From for SpanArithmetic<'static>
fn from(span: &'a Span) -> SpanArithmetic<'static>
impl<'a> RefUnwindSafe for SpanArithmetic<'a>
impl<'a> Send for SpanArithmetic<'a>
impl<'a> Sync for SpanArithmetic<'a>
impl<'a> Unpin for SpanArithmetic<'a>
impl<'a> UnsafeUnpin for SpanArithmetic<'a>
impl<'a> UnwindSafe for SpanArithmetic<'a>
impl<'a, 'b> From for SpanArithmetic<'b>
fn from((span, zoned): (&'a Span, &'b Zoned)) -> SpanArithmetic<'b>
impl<'a, 'b> From for SpanArithmetic<'b>
fn from((span, relative): (&'a Span, SpanRelativeTo<'b>)) -> SpanArithmetic<'b>
impl<T> Any for SpanArithmetic<'a>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for SpanArithmetic<'a>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for SpanArithmetic<'a>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for SpanArithmetic<'a>
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for SpanArithmetic<'a>
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for SpanArithmetic<'a>
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for SpanArithmetic<'a>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for SpanArithmetic<'a>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for SpanArithmetic<'a>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>